###
# Copyright (c) 2011, Rebecca Bettencourt
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

###

import os
import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

class SqlitePronounsDB(object):
    def __init__(self, filename):
        self.dbs = ircutils.IrcDict()
        self.filename = filename

    def close(self):
        for db in self.dbs.itervalues():
            db.close()

    def _getDb(self, channel):
        try:
            import sqlite
        except ImportError:
            raise callbacks.Error, 'You need to have PySQLite installed to ' \
                                   'use Pronouns.  Download it at ' \
                                   '<http://pysqlite.org/>'
        filename = plugins.makeChannelFilename(self.filename, channel)
        if filename in self.dbs:
            return self.dbs[filename]
        if os.path.exists(filename):
            self.dbs[filename] = sqlite.connect(filename)
            return self.dbs[filename]
        db = sqlite.connect(filename)
        self.dbs[filename] = db
        cursor = db.cursor()
        cursor.execute("""CREATE TABLE sgotitle (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          title TEXT
                          )""")
        cursor.execute("""CREATE TABLE sgopronoun (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          pronoun TEXT
                          )""")
        cursor.execute("""CREATE TABLE sgosex (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          sex TEXT
                          )""")
        cursor.execute("""CREATE TABLE sgogender (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          gender TEXT
                          )""")
        cursor.execute("""CREATE TABLE sgopresentation (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          presentation TEXT
                          )""")
        cursor.execute("""CREATE TABLE sgoorientation (
                          id INTEGER PRIMARY KEY,
                          name TEXT,
                          normalized TEXT UNIQUE ON CONFLICT IGNORE,
                          orientation TEXT
                          )""")
        db.commit()
        return db

    def get_title(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT title FROM sgotitle WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_pronoun(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT pronoun FROM sgopronoun WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_sex(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT sex FROM sgosex WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_gender(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT gender FROM sgogender WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_presentation(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT presentation FROM sgopresentation WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def get_orientation(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""SELECT orientation FROM sgoorientation WHERE normalized=%s""", name)
        if cursor.rowcount == 0:
            return None
        else:
            return cursor.fetchone()[0]

    def set_title(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgotitle VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgotitle SET title=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def set_pronoun(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgopronoun VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgopronoun SET pronoun=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def set_sex(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgosex VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgosex SET sex=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def set_gender(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgogender VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgogender SET gender=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def set_presentation(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgopresentation VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgopresentation SET presentation=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def set_orientation(self, channel, name, value):
        db = self._getDb(channel)
        normalized = name.lower()
        cursor = db.cursor()
        cursor.execute("""INSERT INTO sgoorientation VALUES (NULL, %s, %s, '')""", name, normalized)
        cursor.execute("""UPDATE sgoorientation SET orientation=%s WHERE normalized=%s""", value, normalized)
        db.commit()

    def clear_title(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgotitle WHERE normalized=%s""", name)
        db.commit()

    def clear_pronoun(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgopronoun WHERE normalized=%s""", name)
        db.commit()

    def clear_sex(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgosex WHERE normalized=%s""", name)
        db.commit()

    def clear_gender(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgogender WHERE normalized=%s""", name)
        db.commit()

    def clear_presentation(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgopresentation WHERE normalized=%s""", name)
        db.commit()

    def clear_orientation(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgoorientation WHERE normalized=%s""", name)
        db.commit()

    def clear_sgo(self, channel, name):
        db = self._getDb(channel)
        name = name.lower()
        cursor = db.cursor()
        cursor.execute("""DELETE FROM sgotitle WHERE normalized=%s""", name)
        cursor.execute("""DELETE FROM sgopronoun WHERE normalized=%s""", name)
        cursor.execute("""DELETE FROM sgosex WHERE normalized=%s""", name)
        cursor.execute("""DELETE FROM sgogender WHERE normalized=%s""", name)
        cursor.execute("""DELETE FROM sgopresentation WHERE normalized=%s""", name)
        cursor.execute("""DELETE FROM sgoorientation WHERE normalized=%s""", name)
        db.commit()

PronounsDB = plugins.DB('Pronouns', {'sqlite': SqlitePronounsDB})

class Pronouns(callbacks.Plugin):
    """This plugin contains commands for tracking users' pronouns."""
    pass

    def __init__(self, irc):
        self.__parent = super(Pronouns, self)
        self.__parent.__init__(irc)
        self.db = PronounsDB()

    def die(self):
        self.__parent.die()
        self.db.close()

    def title(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your title (Mr., Ms., etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the title of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_title(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_title(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_title(channel, name)
                if value is not None:
                    irc.reply(format('title for %s is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s title', name))
    title = wrap(title, ['channel', many('something')])

    def pronoun(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your pronoun (he, she, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the pronoun of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_pronoun(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_pronoun(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_pronoun(channel, name)
                if value is not None:
                    irc.reply(format('pronoun for %s is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s pronoun', name))
    pronoun = wrap(pronoun, ['channel', many('something')])

    def pronouns(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your pronoun (he, she, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the pronoun of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_pronoun(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_pronoun(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_pronoun(channel, name)
                if value is not None:
                    irc.reply(format('pronouns for %s are %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s pronouns', name))
    pronouns = wrap(pronouns, ['channel', many('something')])

    def sex(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your sex (male, female, intersex, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the sex of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_sex(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_sex(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_sex(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s sex is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s sex', name))
    sex = wrap(sex, ['channel', many('something')])

    def gender(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your gender (male, female, genderqueer, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the gender of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_gender(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_gender(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_gender(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s gender is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s gender', name))
    gender = wrap(gender, ['channel', many('something')])

    def present(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your presentation (masculine, feminine, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the presentation of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_presentation(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_presentation(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_presentation(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s presentation is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s presentation', name))
    present = wrap(present, ['channel', many('something')])

    def presentation(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your presentation (masculine, feminine, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the presentation of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_presentation(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_presentation(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_presentation(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s presentation is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s presentation', name))
    presentation = wrap(presentation, ['channel', many('something')])

    def orient(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your orientation (straight, gay, bi, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the orientation of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_orientation(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_orientation(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_orientation(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s orientation is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s orientation', name))
    orient = wrap(orient, ['channel', many('something')])

    def orientation(self, irc, msg, args, channel, names):
        """[<channel>] [<preposition>] <name> [...]

        If <preposition> is the word "is", "are", or "set", sets your orientation (straight, gay, bi, etc.) to <name>.
        If <preposition> is the word "for", "of", or "get", or is not specified, prints the orientation of each <name>.
        <channel> is only necessary if the message isn't sent in the channel itself.
        """
        if names[0] == 'clear':
            self.db.clear_orientation(channel, irc.msg.nick)
            irc.replySuccess()
        elif names[0] == 'is' or names[0] == 'are' or names[0] == 'set':
            names.remove(names[0])
            self.db.set_orientation(channel, irc.msg.nick, ' '.join(names))
            irc.reply('Ok. :)')
        else:
            if names[0] == 'for' or names[0] == 'of' or names[0] == 'get':
                names.remove(names[0])
            for name in names:
                if name == '.' or name == 'me':
                    name = irc.msg.nick
                value = self.db.get_orientation(channel, name)
                if value is not None:
                    irc.reply(format('%s\'s orientation is %s', name, value))
                else:
                    irc.reply(format('sorry, I don\'t know %s\'s orientation', name))
    orientation = wrap(orientation, ['channel', many('something')])

    def sgo(self, irc, msg, args, channel, name):
        """[<channel>] <name>
        
        Prints the title, pronoun, sex, gender, presentation, and orientation of <name>.
        Only the information currently known about <name> is printed.
        <channel> is only necessary if the message isn't sent in the channel itself.
        Use the individual commands title, pronoun, sex, gender, presentation, and orientation
        to set.
        """
        if name == 'clear':
            self.db.clear_sgo(channel, irc.msg.nick)
            irc.replySuccess()
        else:
            if name == '.' or name == 'me':
                name = irc.msg.nick
            title = self.db.get_title(channel, name);
            pronoun = self.db.get_pronoun(channel, name);
            sex = self.db.get_sex(channel, name);
            gender = self.db.get_gender(channel, name);
            presentation = self.db.get_presentation(channel, name);
            orientation = self.db.get_orientation(channel, name);
            if title is None and pronoun is None and sex is None and gender is None and presentation is None and orientation is None:
                irc.reply(format('sorry, I don\'t have any information on %s', name))
            else:
                if title is not None:
                    irc.reply(format('title for %s is %s', name, title))
                if pronoun is not None:
                    irc.reply(format('pronouns for %s are %s', name, pronoun))
                if sex is not None:
                    irc.reply(format('%s\'s sex is %s', name, sex))
                if gender is not None:
                    irc.reply(format('%s\'s gender is %s', name, gender))
                if presentation is not None:
                    irc.reply(format('%s\'s presentation is %s', name, presentation))
                if orientation is not None:
                    irc.reply(format('%s\'s orientation is %s', name, orientation))
    sgo = wrap(sgo, ['channel', 'something'])

Class = Pronouns

# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
